/* The file is part of Snowman decompiler. */
/* See doc/licenses.asciidoc for the licensing information. */

/* * SmartDec decompiler - SmartDec is a native code to C/C++ decompiler
 * Copyright (C) 2015 Alexander Chernov, Katerina Troshina, Yegor Derevenets,
 * Alexander Fokin, Sergey Levin, Leonid Tsvetkov
 *
 * This file is part of SmartDec decompiler.
 *
 * SmartDec decompiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SmartDec decompiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SmartDec decompiler.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <nc/config.h>
#include <nc/common/Exception.h>

#include <QCoreApplication>

namespace nc { namespace core {

namespace arch {
    class Instruction;
}

namespace irgen {

/**
 * Error info structure for the pointer to the instruction which caused the exception.
 */
typedef boost::error_info<struct InstructionTag, const arch::Instruction *> ExceptionInstruction;

/**
 * Exception that is thrown when an invalid instruction is detected.
 */
class InvalidInstructionException: public Exception {
    Q_DECLARE_TR_FUNCTIONS(InvalidInstructionException)
public:
    /**
     * Constructor.
     *
     * \param message Exception message.
     */
    InvalidInstructionException(const QString &message):
        Exception(message)
    {}

    /**
     * Destructor.
     */
    virtual ~InvalidInstructionException() noexcept {}

    virtual QString unicodeWhat() const noexcept;
};

}}} // namespace nc::core::irgen

/* vim:set et sts=4 sw=4: */
